# 2010 - Copyright by Bee <http://www.honeybeenet.altervista.org>.
# 2012 - Rewritten to work with modern Gedit by Tom Callaway <spot@fedoraproject.org>
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

from gi.repository import GObject, Gtk, Gedit
# import gedit
# import pygtk
# pygtk.require('2.0')
# import gtk
from gettext import gettext as _
import os
import tempfile
import time
import sys
import traceback
import subprocess
import pickle

ui_str = """
<ui>
  <menubar name="MenuBar">
    <menu name="ToolsMenu" action="Tools">
      <placeholder name="ToolsOps_2">
        <menuitem name="beesuPlugin" action="beesuPlugin"/>
      </placeholder>
    </menu>
  </menubar>
</ui>
"""

class beesuPlugin(GObject.Object, Gedit.WindowActivatable):
    __gtype_name__ = "BeesuPlugin"

    window = GObject.property(type=Gedit.Window)

    def __init__(self):
        GObject.Object.__init__(self)

    def do_activate(self):
        self._insert_menu()

    def do_deactivate(self):
        self._remove_menu()

    def _remove_menu(self):
        manager = self.window.get_ui_manager()
        manager.remove_ui(self._ui_id)
        manager.remove_action_group(self._action_group)
        self._action_group = None
        manager.ensure_update()

    def _insert_menu(self):
        manager = self.window.get_ui_manager()

        self._action_group = Gtk.ActionGroup(name="beesuPluginActions")
        self._action_group.add_actions([("beesuPlugin", None,
                                        _("Open as \"root\""),
                                         None,
                                        _("Open this document as \"root\" in gedit with beesu"),
                                         self.beesu)])

        manager.insert_action_group(self._action_group, -1)
        self._ui_id = manager.add_ui_from_string(ui_str)

    def is_configurable(self):
        return False  

    def error_message(message):
        def msg_on_close_destroy(button,response_id,dialog):
            if response_id == Gtk.ResponseType.CLOSE:
                dialog.destroy()

        msg_dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE, message)
        msg_dialog.connect('response', msg_on_close_destroy, msg_dialog)
        msg_dialog.run()


    def beesu(self, action):
        tab = self.window.get_active_tab()
        doc = self.window.get_active_document()

        filename = None
        if doc:
            location = doc.get_location()
            if location:
               uri = location.get_uri()
               if uri is not None:
                   if uri[:7] == "file://":
                       filename = location.get_parse_name()

        if not self.beesu_exec(filename):
            return

        if tab:
            if doc == tab.get_document():
                if doc.get_readonly() or doc.is_untouched():
                    self.window.close_tab(tab)
                    if self.window.get_active_document() == None and self.window.get_active_tab() == None:
                        self.window.destroy()

    def beesu_exec(self, filename):
        tmp = tempfile.NamedTemporaryFile(delete = False)
        lockfile = tmp.name
        tmp.close()
        #shell_args = ['beesu', '-l','/usr/libexec/gedit-beesu-plugin','gedit-beesu-plugin','gedit','gedit',lockfile]
        if filename != None and filename != '':
            # shell_args = ['beesu', '-l', '-c', '\"/usr/bin/gedit %s\"' % filename]
            shell_args = ['beesu', '-l', '-c', '/usr/libexec/gedit-beesu-plugin','gedit-beesu-plugin','gedit','gedit',lockfile, filename]
            # shell_args = ['/usr/libexec/gedit-beesu-plugin','gedit-beesu-plugin','gedit','gedit', lockfile, filename]
        else:
           return False
        print shell_args
        try:
            subprocess.Popen(shell_args, stderr=open(os.devnull)).wait()
        except:
            etype = sys.exc_info()[0]
            evalue = sys.exc_info()[1]
            etb = traceback.extract_tb(sys.exc_info()[2])
            myerror = 'Error running beesu: ' + '\n' + 'Error Type: ' + str(etype) + '\n' + 'Error Value: ' + str(evalue) + '\n' + 'Traceback: ' + str(etb)
            self.error_message(_(myerror))
            return False

        print 'Hey, it says the Popen succeeded.'

        if os.path.isfile(lockfile):
            os.remove(lockfile)
            return False

        return True
